from django.contrib import admin

from mptt.admin import MPTTModelAdmin

from glossary import models


class GlossaryAdmin(admin.ModelAdmin):
    exclude = ("uuid", "deleted", )

    def has_add_permission(self, request):
        return False

    def has_delete_permission(self, request, obj=None):
        return False


@admin.register(models.Faculty)
class FacultyAdmin(GlossaryAdmin):
    readonly_fields = ("name", )


@admin.register(models.Department)
class DepartmentAdmin(GlossaryAdmin):
    readonly_fields = ("name", "faculty")


@admin.register(models.AcademicGroup)
class AcademicGroupAdmin(GlossaryAdmin):
    readonly_fields = ("name", "faculty", "year")
    list_filter = ("faculty", )
    list_display = ("name", "faculty", "year")


@admin.register(models.Address)
class AddressAdmin(GlossaryAdmin):
    pass


@admin.register(models.Campus)
class CampusAdmin(GlossaryAdmin):
    pass


@admin.register(models.RoomType)
class RoomTypeAdmin(GlossaryAdmin):
    pass


@admin.register(models.Room)
class RoomAdmin(GlossaryAdmin):
    pass


@admin.register(models.Subdivision)
class SubdivisionAdmin(MPTTModelAdmin, GlossaryAdmin):
    pass


# -*- coding: utf-8 -*-
# Generated by Django 1.10.6 on 2017-05-04 02:53
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('glossary', '0002_auto_20170314_0850'),
    ]

    operations = [
        migrations.AlterField(
            model_name='academicgroup',
            name='year',
            field=models.SmallIntegerField(choices=[(1, 'Первый'), (2, 'Второй'), (3, 'Третий'), (4, 'Четвертый'), (5, 'Пятый'), (6, 'Шестой')], default=1, verbose_name='Курс'),
        ),
    ]



import uuid

from django.db import models

from glossary.queryset import GlossaryManager


class GlossaryModel(models.Model):
    """
    Набор полей и методов для всех данных, которые
    нужно синхронизировать между основной копией и репликами.

    'uuid' - уникальный идентификатор, не зависящий от конкретной БД.
    'deleted' - метка "удален". Модели-наследники не удаляются
    насовсем, а скрываются при обычных запросах.
    """
    uuid = models.UUIDField(
        editable=False, default=uuid.uuid4, unique=True
    )
    deleted = models.BooleanField(default=False)

    objects = GlossaryManager()
    deleted_objects = GlossaryManager(show_deleted=True)
    all_objects = models.Manager()

    class Meta:
        abstract = True

    def delete(self):
        self.deleted = True
        self.save()

    def force_delete(self):
        return super().delete()

    def dump(self):
        data = {}
        for field in self._meta.fields:
            if isinstance(field, models.AutoField):
                continue
            elif isinstance(field, models.ForeignKey):
                related = getattr(self, field.name)
                related_uuid = str(related.uuid)
                data[field.name] = related_uuid
            elif isinstance(field, models.UUIDField):
                data[field.name] = str(getattr(self, field.name))
            else:
                data[field.name] = getattr(self, field.name)
        return data

    @classmethod
    def clean_dump_data(cls, data):
        for field in cls._meta.fields:
            if isinstance(field, models.AutoField):
                continue
            elif isinstance(field, models.ForeignKey):
                related_model = field.rel.to
                related_uuid = data[field.name]
                related = related_model.all_objects.get(uuid=related_uuid)
                data[field.name] = related
        return data

    @classmethod
    def get_foreign_key_count(cls):
        fk_count = 0
        for field in cls._meta.fields:
            if isinstance(field, models.ForeignKey):
                fk_count += 1
        return fk_count


class Faculty(GlossaryModel):
    name = models.CharField(verbose_name="Название", max_length=127)

    class Meta:
        verbose_name = "Факультет"
        verbose_name_plural = "Факультеты"

    def __str__(self):
        return self.name


class Department(GlossaryModel):
    name = models.CharField(verbose_name="Название", max_length=127)
    faculty = models.ForeignKey(Faculty, verbose_name="Факультет", on_delete=models.CASCADE)

    class Meta:
        verbose_name = "Кафедра"
        verbose_name_plural = "Кафедры"

    def __str__(self):
        return "{} - {}".format(self.faculty, self.name)


class AcademicGroup(GlossaryModel):
    FIRST = 1
    SECOND = 2
    THIRD = 3
    FOURTH = 4
    FIFTH = 5
    SIXTH = 6
    YEAR_CHOICES = (
        (FIRST, "Первый"),
        (SECOND, "Второй"),
        (THIRD, "Третий"),
        (FOURTH, "Четвертый"),
        (FIFTH, "Пятый"),
        (SIXTH, "Шестой"),
    )
    name = models.CharField(verbose_name="Название", max_length=127)
    faculty = models.ForeignKey(Faculty, verbose_name="Факультет", on_delete=models.CASCADE)
    year = models.SmallIntegerField(
        verbose_name="Курс", default=FIRST, choices=YEAR_CHOICES
    )
    dekid = models.IntegerField(verbose_name="ID в Деканате", default=0)

    class Meta:
        verbose_name = "Академическая группа"
        verbose_name_plural = "Академические группы"

    def __str__(self):
        return "{} - {}".format(self.faculty, self.name)



import json

from django.conf import settings
from django.http import JsonResponse
from django.http import HttpResponse
from django.views.decorators.csrf import csrf_exempt

from .utils import apply_update


@csrf_exempt
def update(request):
    if "data" not in request.POST:
        return JsonResponse(
            {"error": "Неправильный формат запроса"},
            status=422
        )
    if "HTTP_AUTHORIZATION" not in request.META:
        return JsonResponse(
            {"error": "Требуется авторизация"},
            status=403
        )

    auth = request.META["HTTP_AUTHORIZATION"]
    required_auth = "Bearer {}".format(settings.GLOSSARY_SERVICE_TOKEN)
    if auth != required_auth:
        return JsonResponse({"error": "Неверный токен"}, status=403)

    data = json.loads(request.POST["data"])
    try:
        apply_update(data)
    except BaseException as e:
        return JsonResponse({"error": str(e)}, status=500)

    return HttpResponse()



from urllib.parse import urljoin

from django.core.management.base import BaseCommand
from django.urls import reverse

from glossary import GlossaryService


class Command(BaseCommand):
    help = 'Регистрация приложения для использования глоссария'

    def add_arguments(self, parser):
        parser.add_argument('host', type=str)

    def handle(self, *args, **options):
        host = options["host"]
        update_url = reverse("glossary-update")
        full_url = urljoin(host, update_url)
        token = GlossaryService.register(full_url)
        self.stdout.write(
            "Добавьте GLOSSARY_SERVICE_TOKEN = '{}' в Django settings"
            .format(token)
        )


from django.db import models


class GlossaryQuerySet(models.QuerySet):
    """
    QuerySet, который не позволяет удалять записи из БД
    пачками. Вместо этого устанавливается метка "удален".

    На записи 'GlossaryModel' так же переопределен метод
    'delete'.
    """

    def delete(self):
        return self.update(deleted=True)

    def force_delete(self):
        return super().delete()


class GlossaryManager(models.Manager):
    """
    Этот менеджер может показывать либо только записи
    без метки "удален", либо только записи с этой меткой.

    За это поведение отвечает параметр 'show_deleted'.
    По умолчанию показываются все записи без метки "удалено".
    """

    def __init__(self, *args, **kwargs):
        self.show_deleted = kwargs.pop("show_deleted", False)
        super().__init__(*args, **kwargs)

    def get_queryset(self):
        return GlossaryQuerySet(self.model, using=self._db)\
            .filter(deleted=self.show_deleted)



# -*- coding: utf-8 -*-
# Generated by Django 1.10.8 on 2019-01-31 09:42
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import mptt.fields
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ('glossary', '0004_academicgroup_dekid'),
    ]

    operations = [
        migrations.CreateModel(
            name='Address',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('uuid', models.UUIDField(default=uuid.uuid4, editable=False, unique=True)),
                ('deleted', models.BooleanField(default=False)),
                ('name', models.TextField(verbose_name='Адрес')),
                ('longitude', models.FloatField(blank=True, null=True, verbose_name='Долгота')),
                ('latitude', models.FloatField(blank=True, null=True, verbose_name='Широта')),
            ],
            options={
                'verbose_name': 'Адрес',
                'verbose_name_plural': 'Адреса',
            },
        ),
        migrations.CreateModel(
            name='Campus',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('uuid', models.UUIDField(default=uuid.uuid4, editable=False, unique=True)),
                ('deleted', models.BooleanField(default=False)),
                ('name', models.CharField(max_length=500, verbose_name='Наименование')),
                ('address', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='glossary.Address', verbose_name='Адрес')),
            ],
            options={
                'verbose_name': 'Здание',
                'verbose_name_plural': 'Здания',
            },
        ),
        migrations.CreateModel(
            name='Room',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('uuid', models.UUIDField(default=uuid.uuid4, editable=False, unique=True)),
                ('deleted', models.BooleanField(default=False)),
                ('name', models.CharField(max_length=500, verbose_name='Наименование/номер')),
                ('description', models.TextField(blank=True, verbose_name='Описание')),
                ('places', models.PositiveSmallIntegerField(blank=True, null=True, verbose_name='Количество посадочных мест')),
                ('size', models.FloatField(blank=True, null=True, verbose_name='Площадь')),
                ('floor', models.PositiveSmallIntegerField(blank=True, choices=[(0, 0), (1, 1), (2, 2), (3, 3), (4, 4), (5, 5), (6, 6), (7, 7), (8, 8), (9, 9)], null=True, verbose_name='Этаж')),
            ],
            options={
                'verbose_name': 'Тип помещения',
                'verbose_name_plural': 'Тип помещений',
            },
        ),
        migrations.CreateModel(
            name='RoomType',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('uuid', models.UUIDField(default=uuid.uuid4, editable=False, unique=True)),
                ('deleted', models.BooleanField(default=False)),
                ('name', models.CharField(max_length=500, verbose_name='Тип')),
            ],
            options={
                'verbose_name': 'Тип помещения',
                'verbose_name_plural': 'Тип помещений',
            },
        ),
        migrations.CreateModel(
            name='Subdivision',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('uuid', models.UUIDField(default=uuid.uuid4, editable=False, unique=True)),
                ('deleted', models.BooleanField(default=False)),
                ('name', models.CharField(max_length=500, verbose_name='Наименование')),
                ('lft', models.PositiveIntegerField(db_index=True, editable=False)),
                ('rght', models.PositiveIntegerField(db_index=True, editable=False)),
                ('tree_id', models.PositiveIntegerField(db_index=True, editable=False)),
                ('level', models.PositiveIntegerField(db_index=True, editable=False)),
                ('parent', mptt.fields.TreeForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='children', to='glossary.Subdivision')),
            ],
            options={
                'verbose_name': 'Подразделение',
                'verbose_name_plural': 'Подразделения',
            },
        ),
        migrations.AddField(
            model_name='room',
            name='subdivisions',
            field=models.ManyToManyField(blank=True, to='glossary.Subdivision', verbose_name='Подразделения'),
        ),
        migrations.AddField(
            model_name='room',
            name='type',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='glossary.RoomType', verbose_name='Тип помещения'),
        ),
    ]



from django.db import models
from glossary.models import GlossaryModel
from mptt.models import MPTTModel, TreeForeignKey


class Address(GlossaryModel):
    name = models.TextField('Адрес')
    longitude = models.FloatField('Долгота', blank=True, null=True)
    latitude = models.FloatField('Широта', blank=True, null=True)

    def __str__(self):
        return str(self.name)

    class Meta:
        verbose_name = 'Адрес'
        verbose_name_plural = 'Адреса'


class Campus(GlossaryModel):
    name = models.CharField('Наименование', max_length=500)
    address = models.ForeignKey(Address, verbose_name='Адрес', on_delete=models.CASCADE)

    def __str__(self):
        return str(self.name)

    class Meta:
        verbose_name = 'Здание'
        verbose_name_plural = 'Здания'


class RoomType(GlossaryModel):
    name = models.CharField('Тип', max_length=500)

    def __str__(self):
        return str(self.name)

    class Meta:
        verbose_name = 'Тип помещения'
        verbose_name_plural = 'Тип помещений'


class Subdivision(MPTTModel, GlossaryModel):
    name = models.CharField('Наименование', max_length=500)
    parent = TreeForeignKey('self', on_delete=models.CASCADE, null=True, blank=True, related_name='children')

    def __str__(self):
        return str(self.name)

    class Meta:
        verbose_name = 'Подразделение'
        verbose_name_plural = 'Подразделения'


class Room(GlossaryModel):
    FLOORS = [(i, i) for i in range(10)]

    name = models.CharField('Наименование/номер', max_length=500)
    type = models.ForeignKey(RoomType, verbose_name='Тип помещения', blank=True, null=True, on_delete=models.CASCADE)
    description = models.TextField('Описание', blank=True)
    places = models.PositiveSmallIntegerField('Количество посадочных мест', blank=True, null=True)
    size = models.FloatField('Площадь', blank=True, null=True)
    floor = models.PositiveSmallIntegerField('Этаж', choices=FLOORS, blank=True, null=True)
    subdivisions = models.ManyToManyField(Subdivision, verbose_name='Подразделения', blank=True)

    def __str__(self):
        return str(self.name)

    class Meta:
        verbose_name = 'Помещение'
        verbose_name_plural = 'Помещения'



# -*- coding: utf-8 -*-
# Generated by Django 1.10.6 on 2017-03-14 08:50
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('glossary', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='academicgroup',
            name='deleted',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='department',
            name='deleted',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='faculty',
            name='deleted',
            field=models.BooleanField(default=False),
        ),
    ]



from urllib.parse import urljoin

import time
import requests

from django.conf import settings


class ApiError(BaseException):
    pass


class GlossaryService(object):

    register_url = urljoin(settings.GLOSSARY_SERVICE_URL, "/register-app")
    json_model_url = urljoin(settings.GLOSSARY_SERVICE_URL, "/get-json-models")

    @classmethod
    def request_log(cls, status_code):
        if status_code != 200:
            raise ApiError(
                "Ошибка при регистрации приложения. Код - {}"
                .format(status_code)
            )

    @classmethod
    def register(cls, update_url):
        r = requests.post(cls.register_url, {"update_url": update_url})
        cls.request_log(r.status_code)
        return r.json()["token"]

    @classmethod
    def get_json_models(cls):
        r = requests.get(cls.json_model_url)
        cls.request_log(r.status_code)
        return r.json()



# Generated by Django 2.1.5 on 2019-02-04 08:53

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('glossary', '0005_auto_20190131_0942'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='room',
            options={'verbose_name': 'Помещение', 'verbose_name_plural': 'Помещения'},
        ),
    ]



from urllib.parse import urljoin
from glossary.utils import apply_update
from django.core.management.base import BaseCommand
from glossary import GlossaryService
import json


class Command(BaseCommand):
    help = 'Запрос обновлений'

    def handle(self, *args, **options):
        data = GlossaryService.get_json_models()
        apply_update(json.loads(data))
        self.stdout.write("Обновления успешно загружены")


# -*- coding: utf-8 -*-
# Generated by Django 1.10.6 on 2017-03-14 06:07
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='AcademicGroup',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('uuid', models.UUIDField(default=uuid.uuid4, editable=False, unique=True)),
                ('name', models.CharField(max_length=127, verbose_name='Название')),
                ('year', models.SmallIntegerField(choices=[(1, 'Первый'), (2, 'Второй'), (3, 'Третий'), (4, 'Четвертый'), (5, 'Пятый')], default=1, verbose_name='Курс')),
            ],
            options={
                'verbose_name_plural': 'Академические группы',
                'verbose_name': 'Академическая группа',
            },
        ),
        migrations.CreateModel(
            name='Department',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('uuid', models.UUIDField(default=uuid.uuid4, editable=False, unique=True)),
                ('name', models.CharField(max_length=127, verbose_name='Название')),
            ],
            options={
                'verbose_name_plural': 'Кафедры',
                'verbose_name': 'Кафедра',
            },
        ),
        migrations.CreateModel(
            name='Faculty',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('uuid', models.UUIDField(default=uuid.uuid4, editable=False, unique=True)),
                ('name', models.CharField(max_length=127, verbose_name='Название')),
            ],
            options={
                'verbose_name_plural': 'Факультеты',
                'verbose_name': 'Факультет',
            },
        ),
        migrations.AddField(
            model_name='department',
            name='faculty',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='glossary.Faculty', verbose_name='Факультет'),
        ),
        migrations.AddField(
            model_name='academicgroup',
            name='faculty',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='glossary.Faculty', verbose_name='Факультет'),
        ),
    ]



from django.apps import apps


def apply_update(data):
    models = apps.get_app_config("glossary").get_models()
    models = sorted(models, key=lambda m: m.get_foreign_key_count())
    for model in models:
        records = data[model._meta.model_name]
        for record in records:
            uuid = record["uuid"]
            obj_data = model.clean_dump_data(record)

            try:
                model.all_objects.get(uuid=uuid)
                model.all_objects.filter(uuid=uuid).update(**obj_data)
            except model.DoesNotExist:
                model.all_objects.create(**obj_data)




from setuptools import setup
from setuptools import find_packages

version = "0.4.1"

setup(
    name='glossary',
    version=version,
    author='dissw-team',
    author_email='lab.dissw@gmail.com',
    maintainer='Dmitry Shlagov',
    maintainer_email='wessmoke@gmail.com',
    description='Glossary with common Ionosphere data',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: Russian',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5'
    ],
    license='MIT',
    packages=find_packages(include=["glossary", "glossary.*"]),
    include_package_data=True,
    install_requires=[
        "setuptools",
        "Django>=1.10.6",
        "requests",
        "django-mptt==0.8.6"
    ],
    zip_safe=False,
)



# -*- coding: utf-8 -*-
# Generated by Django 1.10.8 on 2017-12-18 04:45
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('glossary', '0003_auto_20170504_0253'),
    ]

    operations = [
        migrations.AddField(
            model_name='academicgroup',
            name='dekid',
            field=models.IntegerField(default=0, verbose_name='ID в Деканате'),
        ),
    ]
